# coding: utf-8
"""
Python module DocString
"""
__author__ = 'fattakhov'
import datetime

from dateutil.relativedelta import relativedelta
from django.test import TestCase

from gisdo import constants as const
from gisdo.alchemy_session import session_scope
from gisdo.algorithm.enrolled import (
    get_22_index,
    get_22_1_index,
    get_22_2_index,
    get_22_3_index,
    get_22_4_index,
    get_22_5_index,
    get_22_6_index,
    get_22_7_index,
    get_22_8_index,
    get_30_1_index,
    get_30_2_index)
from gisdo.index import (
    ApplicationIndex,
    EnrolledIndex,
    get_queue_index,
    get_queue_index_collection)
from gisdo.index.enrolled import Index
from gisdo.utils import UnitHelper
from kinder.core.children.tests import factory_child
from kinder.core.declaration.models import DeclarationStatusLog
from kinder.core.declaration.tests import factory_declaration
from kinder.core.declaration.tests import utils as test_utils_declaration
from kinder.core.declaration_status.enum import DSS
from kinder.core.declaration_status.models import DeclarationStatus
from kinder.core.direct.tests import factory_direct
from kinder.core.direct.proxy import DirectModel
from kinder.core.direct.models import DirectStatus, DRS
from kinder.core.dict.models import (
    GroupTypeEnumerate, GroupType, HealthNeed, HNE)
from kinder.core.group.tests import factory_group
from kinder.core.unit.models import Unit
from kinder.core.unit.tests import factory_unit


class UnitHelperTestCase(TestCase):
    """
    Тесты для класса работы с деревом учреждений
    """
    def setUp(self):
        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo_kazan = factory_unit.UnitMoFactory(
            name=u'Казань', parent=self.region)
        mo_kazan.save()
        mo_chelny = factory_unit.UnitMoFactory(
            name=u'Наб. Челны',parent=self.region)
        mo_chelny.save()
        mo_arsk = factory_unit.UnitMoFactory(
            name=u'Арск',parent=self.region)
        mo_arsk.save()
        self.mo_list = [mo_kazan,mo_chelny, mo_arsk]
        status = DeclarationStatus.objects.get(code=DSS.REGISTERED)
        for mo in self.mo_list:
            for dou_i in range(0, 10):
                factory_unit.UnitDouFactory(parent=mo, name=dou_i)

    def test_mo_count(self):
        """Проверки:
        - в список мо для расчета, попадают только мо, под которыми есть доу
        :return:
        """
        self.region.save()
        self.unit_helper = UnitHelper(self.region)
        mo_count = len(list(self.unit_helper.get_report_units()))
        self.failUnlessEqual(len(self.mo_list), mo_count)


class QueueIndexTest(TestCase):
    """
    Тесты для группы показателей "Информация о детях в очереди"
    """

    def setUp(self):
        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo = factory_unit.UnitMoFactory(parent=self.region)
        self.dou = factory_unit.UnitDouFactory(parent=mo)
        self.combi_group = factory_group.FactGroupF(
            name=u'Комбинированная',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.COMBI))
        self.decl_list = []
        for i in range(0,10):
            child = factory_child.ChildF.create()

            decl = factory_declaration.DeclarationF.create(
                children=child, mo=mo,
                status=DeclarationStatus.get_by_code(DSS.DUL_CONFIRMATING))
            factory_declaration.DUnitF(declaration=decl, unit=self.dou)
            self.decl_list.append(decl)


    def test_181(self):
        """
        "Учитываются заявки в статусах ""Зарегистрировано""
        и ""Подтверждение льгот""
        + заявки в других статусах, у которых есть привязанные направления
        в статусах ""Подтверждено"", ""Предложено системой""
        - заявления в статусе ""Желает изменить ДОУ"".
        Дата желаемого зачисления <= 01.09.2015 (без желаемой датой зачисления),
        (01.09.текущего учебного года)
        :return:
        """

        self.assertEquals(0, len(get_queue_index_collection('18.1', self.dou)))
        self.decl_list[0].change_status(
            DeclarationStatus.get_by_code(DSS.REGISTERED))
        self.decl_list[1].change_status(
            DeclarationStatus.get_by_code(DSS.PRIV_CONFIRMATING))
        self.assertEquals(2, len(get_queue_index_collection('18.1', self.dou)))

        self.decl_list[3].change_status(
            DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU))
        self.assertEquals(2, len(get_queue_index_collection('18.1', self.dou)))
        direct = factory_direct.DirectFactory(
            group=self.combi_group, declaration=self.decl_list[4])
        self.assertEquals(3, len(get_queue_index_collection('18.1', self.dou)))
        DirectModel.change_status(direct,
                                  DirectStatus.objects.get(code=DRS.CONFIRM))
        self.assertEquals(3, len(get_queue_index_collection('18.1', self.dou)))


class CapacitiesIndexTest(TestCase):
    """
    Тесты для группы показателей "Информация о свободных местах"
    """
    def setUp(self):
        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo = factory_unit.UnitMoFactory(parent=self.region)
        self.dou = factory_unit.UnitDouFactory(parent=mo)
        self.decl_list = []
        for i in range(0,10):
            child = factory_child.ChildF.create()

            decl = factory_declaration.DeclarationF.create(
                children=child, mo=mo,
                status=DeclarationStatus.get_by_code(DSS.DUL_CONFIRMATING))
            factory_declaration.DUnitF(declaration=decl, unit=self.dou)
            self.decl_list.append(decl)


    def test_33_index(self):
        """
        Передается численность детей, снятых с учета в течение
        **текущего календарного года** по таким причинам:

        1. Не явился (код didnt_come)
        2. Отказано в услуге (код refused)
            в случае изменения заявления со статуса
            Зарегистрировано, Желает изменить ДОУ на статус
            Отказано в услуге
        3. Архивное (код archive)
            в случае изменения заявления со статуса
            Зарегистрировано на статус Архивное
        4. Архивное (код archive)
            в случае изменения заявления со статуса
            Желает изменить ДОУ на Архивное

        При расчета показателя 33 необходимо учитывать все заявления,
        у которых дата изменения статуса заявления входит в период с 01.01.хххх,
        где хххх - текущий год.
        :return:
        """

        self.assertEquals(0, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)
        # 1
        self.decl_list[0].change_status(
            DeclarationStatus.get_by_code(DSS.DIDNT_COME))
        self.assertEquals(1, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)
        # 2
        self.assertEquals(self.decl_list[1].status.code, DSS.DUL_CONFIRMATING,
                          u'У заявления неверный статус')
        self.decl_list[1].change_status(
            DeclarationStatus.get_by_code(DSS.REFUSED))
        self.assertEquals(1, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)

        self.decl_list[2].change_status(
            DeclarationStatus.get_by_code(DSS.REGISTERED))
        self.decl_list[2].change_status(
            DeclarationStatus.get_by_code(DSS.REFUSED))
        self.assertEquals(2, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)


        self.decl_list[3].change_status(
            DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU))
        self.decl_list[3].change_status(
            DeclarationStatus.get_by_code(DSS.REFUSED))
        self.assertEquals(3, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)

        # 3
        self.decl_list[4].change_status(
            DeclarationStatus.get_by_code(DSS.ARCHIVE))
        self.assertEquals(3, get_queue_index("33", self.dou),
                          u'Кол-во не должно изменится, тк не выполнено условие'
                          u'что статус был "Зарегистрировано"' )

        self.decl_list[4].change_status(
            DeclarationStatus.get_by_code(DSS.REGISTERED))
        self.decl_list[4].change_status(
            DeclarationStatus.get_by_code(DSS.ARCHIVE))
        self.assertEquals(4, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)
        # 4
        self.decl_list[5].change_status(
            DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU))
        self.decl_list[5].change_status(
            DeclarationStatus.get_by_code(DSS.ARCHIVE))
        self.assertEquals(5, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)

        # проверка на дату изменения
        DeclarationStatusLog.objects.filter(
            status=DeclarationStatus.get_by_code(DSS.ARCHIVE)).update(
            datetime=datetime.datetime.now() - relativedelta(years=1))
        self.assertEquals(3, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)


class ApplicationIndexTest(TestCase):
    """
    Тесты для группы показателей "Информация о заявлениях"
    """

    def test_sum_age_slice_equal_sum_delivery_slice(self):
        """
        Проверяем, чтобы сумма количества заявлений в разрезе возрастных
        категорий была равно сумме количества заявлений в разрезе типа подачи
        """

        units = Unit.objects.filter(kind__id=4)

        with session_scope() as session:
            for dou in units:
                index = ApplicationIndex(dou, session)
                sum_in_age_slice = 0
                for age in const.AGE_CATEGORIES_FULL.values():
                    sum_in_age_slice += index.get_count(age_range=age)
                sum_in_delivery_slice = 0
                sum_in_delivery_slice += (index.get_count(
                    index_type=ApplicationIndex.COUNT_BY_DELIVERY,
                    age_range=age))
                sum_in_delivery_slice += (index.get_count(
                    index_type=ApplicationIndex.COUNT_BY_DELIVERY,
                    portal=1,
                    age_range=age))
                self.assertEquals(sum_in_age_slice, sum_in_delivery_slice,
                                  u'Ошибка по организации %s' % dou.name)


class EnrolledIndexTest(TestCase):
    """
    Тесты для группы показателей "Информация о зачисленных детях"
    Важно! У класса gisdo.index.enrolled.Index cписок групп кешируется,
    поэтому при изменение группы неоходимо создавать новый инстанс индекса
    """
    def setUp(self):
        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo = factory_unit.UnitMoFactory(parent=self.region)
        self.dou = factory_unit.UnitDouFactory(parent=mo)
        self.child_list = []
        for i in range(0,10):
            self.child_list.append(factory_child.ChildF.create())

        self.comp_group = factory_group.FactGroupF(
            name=u'Компенсирующая',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.COMP))
        self.health_group = factory_group.FactGroupF(
            name=u'Оздоровительная',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.HEALTH))
        self.combi_group = factory_group.FactGroupF(
            name=u'Комбинированная',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.COMBI))

    def test_191(self):
        """
        Количество детей в текущих группах.
         Временно зачисленные учитываются, временно отчисленные нет.
        :return:
        """
        enrolled_index = Index(self.dou)
        index_19 = list(enrolled_index(force=True))
        self.assertEquals(
            0, len(index_19), u'Показатель 19 не 0')
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        factory_group.PupilF(grup=self.health_group, temporary=True,
                             children=self.child_list[2])

        self.assertEquals(3, len(list(enrolled_index(force=True))))

    def test_22(self):
        """
        Общая численность детей, зачисленных в группы для детей с ОВЗ.
        Количество детей в текущих группах с направленностью "Комбинированноая"
         или/и "Компенсирующая".Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_index(enrolled_index)), u'Показатель 22 не 0')


        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        factory_group.PupilF(grup=self.health_group,
                             children=self.child_list[2])

        self.assertEquals(2, len(get_22_index(enrolled_index)))

    def test_221(self):
        """
        Передается общая численность детей с ОВЗ, зачисленных в
        группы компенсирующей и комбинированной направленности для детей с
        нарушением слуха.
        Количество детей в группе "Компенсирующая" с овз = с нарушением слуха
        (deafness, hardofhearing). Дети в комбинированных группа
        с овз =(deafness, hardofhearing) у которых в карточке ребенка есть овз
        (если у ребенка выбран ОВЗ оздор. типа эти дети попадают в показатель
        только при наличии заполненного поля Документ, подтверждающий ОВЗ)
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_1_index(enrolled_index)), u'Показатель 22.1 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.DEAFNESS)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_1_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.HARDOFHEARTING)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.HARDOFHEARTING)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_1_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_1_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_1_index(enrolled_index)))

    def test_222(self):
        """
        Количество детей в группе "Компенсирующая" с овз = с нарушением речи
        (speach, phonetics). Дети в комбинированных группа
        с овз =(speach, phonetics) у которых в карточке ребенка есть овз
        (если у ребенка выбран ОВЗ оздор. типа эти дети попадают в показатель
        только при наличии заполненного поля Документ, подтверждающий ОВЗ)
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_2_index(enrolled_index)), u'Показатель 22.2 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.SPEACH)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_2_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.PHONETICS)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.PHONETICS)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_2_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_2_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_2_index(enrolled_index)))

    def test_223(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей с нарушением зрения(blindness, AMAUROSIS).
        Дети в комбинированных группа с овз =  (blindness, AMAUROSIS)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_3_index(enrolled_index)), u'Показатель 22.3 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.BLINDNESS)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_3_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.AMAUROSIS)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.AMAUROSIS)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_3_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_3_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_3_index(enrolled_index)))

    def test_224(self):
        """
        Количество детей в группе "Компенсирующая" с овз = с нарушением интел.
        (backwardnesslight, backwardnesshard). Дети в комбинированных группа
        с овз =(backwardnesslight, backwardnesshard)
        у которых в карточке ребенка есть овз
        (если у ребенка выбран ОВЗ оздор. типа эти дети попадают в показатель
        только при наличии заполненного поля Документ, подтверждающий ОВЗ)
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_4_index(enrolled_index)), u'Показатель 22.4 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.BACKHARD)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_4_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.BACKLIGHT)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.BACKLIGHT)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_4_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_4_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_4_index(enrolled_index)))

    def test_225(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
         для детей с задержкой психического развития.(backwardness).
        Дети в комбинированных группа с овз =  (backwardness)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_5_index(enrolled_index)), u'Показатель 22.5 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.BACK)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_5_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.BACK)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.BACK)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_5_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_5_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_5_index(enrolled_index)))

    def test_226(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей с нарушением опорно-двигательного аппарата (disablement)
        Дети в комбинированных группа с овз =  (disablement)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_6_index(enrolled_index)), u'Показатель 22.6 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.DISABLEMENT)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_6_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.DISABLEMENT)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.DISABLEMENT)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_6_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_6_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_6_index(enrolled_index)))

    def test_227(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей со сложным дефектом(invalidity).
        Дети в комбинированных группа с овз =  (invalidity)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_7_index(enrolled_index)), u'Показатель 22.7 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.INVALIDITY)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_7_index(enrolled_index)))

        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.INVALIDITY)
        self.combi_group.save()
        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.INVALIDITY)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_7_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_7_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_7_index(enrolled_index)))

    def test_228(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей с иными ограниченными возможностями здоровья.
        Дети в комбинированных группа с овз, у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_8_index(enrolled_index)), u'Показатель 22.8 не 0')

        self.comp_group.health_need = HealthNeed.get_by_code(HNE.RESTRICTION)
        self.comp_group.save()
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_8_index(enrolled_index)))

        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.DEAFNESS)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_8_index(enrolled_index)))

        #для этих ОВЗ только при наличие документа
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_8_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_8_index(enrolled_index)))

    def test_301(self):
        """
        Численность детей-инвалидов, имеющих ограниченные возможности здоровья.

        Передается численность детей-инвалидов, зачисленных в ДОО,
        имеющих ограниченние возможностей здоровья, посещающих группы любой
        направленности (т.е. считаем всех детей с чек-боксов ребенок-инвалид
        во всех группах).
        При этом, если у ребенка выбран ОВЗ типа "часто болеющие",
        "с туберкулезной интоксикацией",
        "другие, с необходимостью комплексных оздоровительных мероприятий" ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ.
        :return:
        """

        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_30_1_index(enrolled_index)), u'Показатель 30.1 не 0')


        # В показателе только дети с инвалидностью
        for ch in self.child_list:
            ch.is_invalid = True
            ch.save()
        # зачислим первого ребенка в компенс. группу
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(
            1, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в компенсирующих группах с '
            u'инвалидностью')

        #зачислим второго ребенка в оздоровительную группу
        child = self.child_list[1]
        factory_group.PupilF(grup=self.health_group,
                             children=child)
        self.assertEquals(
            1, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            1, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            2, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        # в группах с типом отличным от оздор. и компенс.
        # проверяется наличие ОВЗ
        child = self.child_list[2]
        factory_group.PupilF(grup=self.combi_group, children=child)
        # С ОВЗ для оздоровитеьных групп, но с типом не оздоровительная,
        # ребенок не попадет в показатель
        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            2, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            3, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп, '
            u'либо с подтверждением')

        child = self.child_list[3]
        factory_group.PupilF(grup=self.combi_group, children=child)
        child.health_need = HealthNeed.get_by_code(HNE.DEAFNESS)
        child.save()
        self.assertEquals(
            4, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')



    def test_302(self):
        """
        Численность детей, не имеющих инвалидности,
        но имеющих ограниченные возможности здоровья.
        Передается численность детей, зачисленных в ДОО,
        не имеющих инвалидности, но имеющих ограниченныых возможностей
        здоровья, посещающих группы любой направленности.
        При этом, если у ребенка выбран ОВЗ типа "часто болеющие",
        "с туберкулезной интоксикацией",
        "другие, с необходимостью комплексных оздоровительных мероприятий" ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ
        Отличие от тестов на 30.1 только отсутсвием признака инвалид детей
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_30_2_index(enrolled_index)), u'Показатель 30.2не 0')


        # зачислим первого ребенка в компенс. группу
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(
            1, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в компенсирующих группах с '
            u'инвалидностью')

        #зачислим второго ребенка в оздоровительную группу
        child = self.child_list[1]
        factory_group.PupilF(grup=self.health_group,
                             children=child)
        self.assertEquals(
            1, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            1, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            2, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        # в группах с типом отличным от оздор. и компенс.
        # проверяется наличие ОВЗ
        child = self.child_list[2]
        factory_group.PupilF(grup=self.combi_group, children=child)
        # С ОВЗ для оздоровитеьных групп, но с типом не оздоровительная,
        # ребенок не попадет в показатель
        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            2, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            3, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп, '
            u'либо с подтверждением')

        child = self.child_list[3]
        factory_group.PupilF(grup=self.combi_group, children=child)
        child.health_need = HealthNeed.get_by_code(HNE.DEAFNESS)
        child.save()
        self.assertEquals(
            4, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

    def test_sum_disabled_group_equals_sum_correct_and_health_groups(self):
        """
        Сумма показателей "Количества детей в группах для детей с
        ограниченными возможностями" в разрезе возрастных
        категорий должно равняться сумме "Количества детей
         в оздоровительных группах" и
        "Количество детей в компенсирующих группах
        """

        units = Unit.objects.filter(kind__id=4)

        with session_scope() as session:
            for dou in units:
                index = EnrolledIndex(dou, session)
                sum_in_disabled_group_age_slice = 0
                for age in const.AGE_CATEGORIES_FULL.values():
                    sum_in_disabled_group_age_slice += index.get_count(
                        age_range=age,
                        index_type=EnrolledIndex.COUNT_DISABLED).count()
                sum_in_correct_and_health_groups = 0
                sum_in_correct_and_health_groups += index.get_count(
                    index_type=EnrolledIndex.COUNT_CORRECT).count()
                sum_in_correct_and_health_groups += index.get_count(
                    index_type=EnrolledIndex.COUNT_HEALTH).count()

                self.assertEqual(sum_in_disabled_group_age_slice,
                                 sum_in_correct_and_health_groups,
                                 u'Ошибка по организации %s' % dou.name)
